////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.foursquare;

import java.net.URLEncoder;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.watto.program.android.sync.foursquare.authenticator.FoursquareAuthenticator;


/***********************************************************************************************
 Provides utility methods for communicating with the backend server
***********************************************************************************************/
public class FoursquareQuery {

  /***********************************************************************************************
  Fetches the list of friend data updates from the server
  @return list The list of updates received from the server.
  ***********************************************************************************************/
  public static FoursquareFriend[] getFriends(){
    try {

      if (!FoursquareAuthenticator.credentialsKnown()) {
        // The authentication is invalid, so return nothing
        return new FoursquareFriend[0];
      }

      String friendsURL = "https://api.foursquare.com/v2/users/self/friends";
      // Sign the request with the access token
      friendsURL = FoursquareAuthenticator.signRequest(friendsURL);
      HttpGet request = new HttpGet(friendsURL);

      // Send the request
      DefaultHttpClient client = new DefaultHttpClient();
      HttpResponse response = client.execute(request);

      // Get the response code
      if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
        // Success
        String responseMessage = EntityUtils.toString(response.getEntity());

        /*
        JSONObject responseWrapper = new JSONObject(responseMessage);
        responseWrapper = responseWrapper.getJSONObject("response");
        responseWrapper = responseWrapper.getJSONObject("friends");
        JSONArray friendArray = responseWrapper.getJSONArray("items");
        */
        JSONArray friendArray = new JSONObject(responseMessage).getJSONObject("response").getJSONObject("friends").getJSONArray("items");

        int friendCount = friendArray.length();
        FoursquareFriend[] friends = new FoursquareFriend[friendCount];

        // Create the friends
        for (int i = 0;i < friendCount;i++) {
          friends[i] = new FoursquareFriend(friendArray.getJSONObject(i));
        }

        // get further details about each friend
        getFriendsDetails(friends);

        return friends;
      }

      return new FoursquareFriend[0];
    }
    catch (Throwable t) {
      // An error occurred in the response, or there were parsing errors
      return new FoursquareFriend[0];
    }
  }


  /***********************************************************************************************
  Fetches further details for a list of <code>FoursquareFriend</code>s from the server
  @param friends The friends being queried
  ***********************************************************************************************/
  public static void getFriendsDetails(FoursquareFriend[] friends){
    try {

      int friendCount = friends.length;

      // Build a multi-query to get the user details of all the friends.
      // FourSquare only allows us to make 5 queries at a time to the 'multi' API call!

      // Work out how many blocks of five are required
      int fiveBlocks = friendCount / 5;
      if (friendCount % 5 != 0) {
        fiveBlocks++;
      }

      int friendsRead = 0;
      for (int b = 0;b < fiveBlocks;b++) {
        String queryBlock = "";

        int friendStart = friendsRead;

        for (int f = 0;f < 5;f++) {
          // get the userID
          long userId = friends[friendsRead].getUserID();

          // Append a query to get the details of this user, so we can grab 
          // the details of all friend 'users' at once (maximum of 5 queries in a block)
          if (f != 0) {
            queryBlock += ","; // comma-separated queries
          }
          queryBlock += URLEncoder.encode("/users/" + userId);
          friendsRead++;

          if (friendsRead >= friendCount) {
            // break when we've finished reading all the friends
            break;
          }
        }

        // second-encode the query block, and sign the request
        queryBlock = "https://api.foursquare.com/v2/multi?requests=" + URLEncoder.encode(queryBlock);

        JSONObject multiResponse = queryServer(queryBlock);
        if (multiResponse == null) {
          // something wrong with this query - try the next batch of 5 users
          continue;
        }

        // loop over each friend and add the extra user details
        JSONArray responsesArray = multiResponse.getJSONObject("response").getJSONArray("responses");

        for (int f = 0;f < 5;f++) {
          // get the friend that we're looking at
          FoursquareFriend friend = friends[friendStart];

          // Get the user
          JSONObject userWrapper = null;
          try {
            userWrapper = responsesArray.getJSONObject(f).getJSONObject("response").getJSONObject("user");
          }
          catch (JSONException e) {
            // Error finding the details for this user, so skip to the next user
            continue;
          }

          // Populate the contact information
          try {
            JSONObject contactWrapper = userWrapper.getJSONObject("contact");
            friend.setPhoneNumber(contactWrapper.optString("phone",""));
            friend.setEmailAddress(contactWrapper.optString("email",""));
            friend.setFacebook(contactWrapper.optString("facebook",""));
            friend.setTwitter(contactWrapper.optString("twitter",""));
          }
          catch (JSONException e) {
            // Ignore any errors trying to set the contact information
          }

          // Populate the last check-in location
          try {
            JSONArray checkinsArray = userWrapper.getJSONObject("checkins").getJSONArray("items");

            if (checkinsArray.length() >= 1) {
              JSONObject checkinWrapper = checkinsArray.getJSONObject(0);
              friend.setCheckinTime(checkinWrapper.optLong("createdAt",0));
              friend.setCheckinTimeZone(checkinWrapper.optString("timeZone",""));
              friend.setShoutout(checkinWrapper.optString("shout",""));

              JSONObject venueWrapper = checkinsArray.getJSONObject(0).getJSONObject("venue");
              friend.setCheckin(venueWrapper.optString("name",""));
            }
          }
          catch (JSONException e) {
            // Ignore any errors trying to set the check-in information
          }

          friendStart++;
          
          if (friendStart >= friendCount) {
            // break when we've finished reading all the friends
            break;
          }
          
        }

      }

    }
    catch (Throwable t) {
      // Error in the response - maybe the server is down or the response message has changed
    }
  }


  /***********************************************************************************************
  Signs a query, sends it to the server, receives a response and converts it to a <code>JSONObject</code> 
  @param queryUrl the URL of the query you wish to perform
  @return the response as a <code>JSONObject</code>, or <b>null</b> if there was an error
  ***********************************************************************************************/
  public static JSONObject queryServer(String queryUrl){
    try {

      // Check that we have valid authentication
      if (!FoursquareAuthenticator.credentialsKnown()) {
        return null;
      }

      // Sign the request with the access token
      queryUrl = FoursquareAuthenticator.signRequest(queryUrl);
      HttpGet request = new HttpGet(queryUrl);

      // Send the request
      DefaultHttpClient client = new DefaultHttpClient();
      HttpResponse response = client.execute(request);

      // Get the response code
      if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
        // Success
        String responseMessage = EntityUtils.toString(response.getEntity());

        // return the JSONObject for the response
        return new JSONObject(responseMessage);
      }
      return null;
    }
    catch (Throwable t) {
      return null;
    }
  }

}
