////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.sync.foursquare.security;

import java.io.IOException;
import java.net.URLEncoder;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpParams;
import org.apache.http.util.EntityUtils;
import org.json.JSONException;
import org.json.JSONObject;


/***********************************************************************************************
Convenience methods for establishing OAuth 2.0 authentication to Foursquare
***********************************************************************************************/
public class FoursquareOAuth2Client extends OAuth2Client {

  /***********************************************************************************************
  Initialises the OAuth provider with the <i>consumerKey</i>, <i>consumerSecret</i> and the 
  <i>callbackUrl</i> for the application
  @param consumerKey the OAuth key for this application
  @param consumerSecret the OAuth secret for this application
  @param callbackUrl the OAuth callback URL for this application
  ***********************************************************************************************/
  public FoursquareOAuth2Client(String consumerKey, String consumerSecret, String callbackUrl){
    super(consumerKey,consumerSecret,callbackUrl);
  }


  /***********************************************************************************************
  Gets the URL for converting an authentication <i>code</i> into an access token
  @param code the authentication code
  @return the URL used to generate the access token
  ***********************************************************************************************/
  public String getAccessTokenUrl(String code){
    setCode(code);
    String encodedCallbackUrl = URLEncoder.encode(callbackUrl);
    return "https://foursquare.com/oauth2/access_token?client_id=" + consumerKey + "&client_secret=" + consumerSecret + "&grant_type=authorization_code&redirect_uri=" + encodedCallbackUrl + "&code=" + code + "&display=touch";
  }


  /***********************************************************************************************
  Gets the URL for authenticating the application
  @return the URL used to authenticate the application
  ***********************************************************************************************/
  public String getAuthenticationUrl(){
    String encodedCallbackUrl = URLEncoder.encode(callbackUrl);
    return "https://foursquare.com/oauth2/authenticate?client_id=" + consumerKey + "&response_type=code&redirect_uri=" + encodedCallbackUrl + "&display=touch";
  }


  /***********************************************************************************************
  Converts the authentication <i>code</i> into an access token
  @param code the authentication code
  @return the access token
  ***********************************************************************************************/
  public String retreiveAccessToken(String code){
    String tokenURL = getAccessTokenUrl(code);

    // Build a HttpClient that ignores SSL errors
    SchemeRegistry schemeRegistry = new SchemeRegistry();
    schemeRegistry.register(new Scheme("http",PlainSocketFactory.getSocketFactory(),80));
    schemeRegistry.register(new Scheme("https",new EasySSLSocketFactory(),443));

    HttpParams params = new BasicHttpParams();
    ClientConnectionManager clientConnectionManager = new ThreadSafeClientConnManager(params,schemeRegistry);

    DefaultHttpClient client = new DefaultHttpClient(clientConnectionManager,params);
    HttpGet get = new HttpGet(tokenURL);

    // Call the URL
    HttpResponse response = null;
    try {
      response = client.execute(get);
    }
    catch (ClientProtocolException e) {}
    catch (IOException e) {}

    String token = "";
    if (response == null || response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
      // error
      token = "";
    }
    else {
      // success - got a token
      try {
        String responseMessage = EntityUtils.toString(response.getEntity());
        JSONObject accessTokenObject = new JSONObject(responseMessage);
        token = accessTokenObject.getString("access_token");
      }
      catch (ParseException e) {}
      catch (IOException e) {}
      catch (JSONException e) {}
    }

    setAccessToken(token);
    return token;

  }


  /***********************************************************************************************
  Signs a URL request with the access token
  @param url the request to sign
  @return the signed request
  ***********************************************************************************************/
  public String signRequest(String url){
    if (url.indexOf('?') > 0) {
      // already has a '?', so need a '&' instead
      return url + "&oauth_token=" + getAccessToken();
    }
    else {
      return url + "?oauth_token=" + getAccessToken();
    }
  }

}
